/*	Read TGA file version 1.1
	Joseph Wang	<http://www.columbia.edu/~jw168>
	2/15/96
	This reads very basic 24-bit targa files output by POV Ray.
*/

#define __STDC__		1
#define __GNU_LIBRARY__ 1

#include <getopt.h>

BFile file;
char input[1024];

class View : public BView {
	BBitmap *bitmap;
	ushort w, h;
	ulong *buffer;
	BRect boundry;
public:
	View(BRect bounds) : BView(bounds, NULL, B_FOLLOW_LEFT, B_WILL_DRAW) {
		boundry=bounds;
		w = boundry.right+1;
		h = boundry.bottom+1;
		bitmap=new BBitmap(boundry, B_RGB_32_BIT);
		buffer=new ulong[w*h];
		buffer=(ulong *)bitmap->Bits();
		}
	void Draw(BRect update) {
		DrawBitmap(bitmap, boundry, boundry);
		}
	void Play() {	//read picture data
		if(file.Open(B_READ_ONLY)==B_NO_ERROR) {
			int error;
			uchar *raw = new uchar[w*h*3];
			file.Seek(21, B_SEEK_TOP);
			error=file.Read(raw, w*h*3);
			if(error<1) return;
			for(int i=0; i<w*h*3; i+=3) {
				uchar r,g,b;
				r=raw[i];
				g=raw[i+1];
				b=raw[i+2];
				buffer[i/3]=b*0x01000000+g*0x010000+r*0x0100;
				}
			Draw(boundry);
			file.Close();
			}
		}
	};

class Win : public BWindow {
	View *view;
public:
	Win(ushort w, ushort h) : BWindow(BRect(0,0,w, h),"TargaView", B_TITLED_WINDOW, B_NOT_RESIZABLE) {
		view=new View(BRect(0,0,w,h));
		Lock();
		AddChild(view);
		Unlock();
		MoveTo(200,200);
		Lock();
		view->Play();
		Unlock();
		Show();
		}
	void MessageReceived(BMessage *msg) {
		switch(msg->what) {
			default:
				MessageQueue()->RemoveMessage(msg);
			}
		}
	bool QuitRequested() {
		be_app->PostMessage(B_QUIT_REQUESTED);
		return TRUE;
		}
	};

class App : public BApplication {
	Win *win;
public:
	App() : BApplication('TGAa') {	//get picture relative to bin
		record_ref ref;
		app_info info;
        be_app->GetAppInfo(&info);
        BDirectory dir;
        BFile *code=new BFile();
        code->SetRef(info.ref);
        code->GetParent(&dir);
        delete code;
		dir.GetRefForPath(input, &ref);
		file.SetRef(ref);
		}

	void ReadyToRun() {
		uchar w_low, h_low, w_high, h_high;
		ushort w, h;
		if(file.Open(B_READ_ONLY)==B_NO_ERROR) {	//read width & height
			file.Seek(12, B_SEEK_TOP);
			file.Read(&w_low, 1);
			file.Read(&w_high, 1);
			file.Read(&h_low, 1);
			file.Read(&h_high, 1);
			w = w_high<<8;		//was this made for a Z80?
			w |= w_low;
			h = h_high<<8;
			h |= h_low;
			win=new Win(w-1, h-1);
			file.Close();
			}
		}

	bool QuitRequested() {return TRUE;}
	};

void main(short argc, char **argv) {
	long t;						//drag n drop
	while((t=getopt(argc,argv,"shxpm8irv:f:l:d:")) != EOF );
	strcpy(input,argv[optind]);
	App app;
	app.Run();
	}
